/*****************************************************************************/
/*!
    @file     	FMRX.cpp
    @author   	www.elechouse.com
	@version  	V1.0
	@date		2012-11-1
	@brief    	FMRX demo source file

	@section  HISTORY
	
    V1.0    Initial version.

    Copyright (c) 2012 www.elechouse.com  All right reserved.
*/
/*****************************************************************************/


#include "FMRX.h"

/**
	TWAR
	TWSR
	TWCR
	TWBR
	TWDR
*/

void i2c_init(void)
{
    /** prescaler = 1 */
    TWSR = 0x01;
    /** baud rate = 0x03 */
    TWBR = 0x03;

    // activate internal pullups for twi.
    digitalWrite(SDA, 1);
    digitalWrite(SCL, 1);

    // initialize twi prescaler and bit rate
    cbi(TWSR, TWPS0);
    cbi(TWSR, TWPS1);
    TWBR = ((F_CPU / I2C_FREQ) - 16) / 2;

    /* twi bit rate formula from atmega128 manual pg 204
    SCL Frequency = CPU Clock Frequency / (16 + (2 * TWBR))
    note: TWBR should be 10 or higher for master mode
    It is 72 for a 16mhz Wiring board with 100kHz TWI */

}
void i2c_start(void)
{
    TWCR = (1<<TWSTA) | (1<<TWINT) | (1<<TWEN);
    while(!(TWCR&(1<<TWINT)));
#ifdef __FM_DEBUG
    Serial.print("I2C Start\r\n");
#endif
    delay(1);
}
void i2c_stop(void)
{
    TWCR = (1<<TWSTO) | (1<<TWINT) | (1<<TWEN);
#ifdef __FM_DEBUG
    Serial.print("I2C Stop\r\n");
#endif
    delay(1);
}

void i2c_write(u8 dt)
{
    while(!(TWCR&(1<<TWINT)));
    TWDR = dt;
    TWCR = (1<<TWINT) | (1<<TWEN);
    while(!(TWCR&(1<<TWINT)));
#ifdef __FM_DEBUG
    Serial.print("I2C Write\r\n");
#endif
    delay(1);
}

u8 i2c_read_ack(void)
{
    while(!(TWCR&(1<<TWINT)));
    TWCR = (1<<TWINT) | (1<<TWEN) | (1<<TWEA);
    while(!(TWCR&(1<<TWINT)));
#ifdef __FM_DEBUG
    Serial.print("I2C Read Ack\r\n");
#endif
    delay(1);
    return TWDR;
}

u8 i2c_read_nack(void)
{
    while(!(TWCR&(1<<TWINT)));
    TWCR = (1<<TWINT) | (1<<TWEN);
    while(!(TWCR&(1<<TWINT)));
#ifdef __FM_DEBUG
    Serial.print("I2C Read Nack\n");
#endif
    delay(1);
    return TWDR;
}


u8 fmrx_reg_w[16] =
{
    0,
    0xd0,0x00,  // 02H
    0x00,0x00,  // 03H
    0x00,0x40,  // 04H
    0x90,0x88,  // 05H
};
u8 fmrx_reg_r[16];
u8 fmrx_band;

void fmrx_write_reg(u8 *reg_buf)
{
#ifdef __RDA5807_MODE
    u8 i;
    i2c_start();
    i2c_write(0x20);
    for(i=1; i<9; i++)
    {
        i2c_write(fmrx_reg_w[i]);
    }
    i2c_stop();
#else
    u8 i;
    i2c_start();
    i2c_write(FMRX_CMD_WRITE);
    for(i=1; i<=5; i++)
    {
        i2c_write(reg_buf[i]);
    }
    i2c_stop();
#endif
}

void fmrx_write_byte(u8 reg, u8 dt)
{
#ifdef __RDA5807_MODE
    fmrx_reg_w[reg] = dt;
    fmrx_write_reg(fmrx_reg_w);
#else
    fmrx_reg_w[reg] = dt;
    fmrx_write_reg(fmrx_reg_w);
#endif
}

void fmrx_read_reg(u8 *reg_buf)
{
#ifdef __RDA5807_MODE
    u8 i;
    i2c_start();
    i2c_write(FMRX_CMD_READ);
    for(i=1; i<4; i++)
    {
        reg_buf[i] = i2c_read_ack();
    }
    reg_buf[4] = i2c_read_nack();
    i2c_stop();
#else
    u8 i;
    i2c_start();
    i2c_write(FMRX_CMD_READ);
    for(i=1; i<5; i++)
    {
        reg_buf[i] = i2c_read_ack();
    }
    reg_buf[5] = i2c_read_nack();
    i2c_stop();
#endif
}

float fmrx_set_freq(float rf_freq)
{
#if 0
    u16 freq;
    rf_freq += 0.225;
    rf_freq *= 4;
    rf_freq /= 0.032768;

    freq = (u16)rf_freq;

    freq &= 0x3FFF;

    fmrx_reg_w[1] = (fmrx_reg_w[1]&0xC0) | (u8)(freq>>8);
    fmrx_reg_w[2] = (u8)freq;
//    fmrx_reg_w[1] = 0;
//    fmrx_reg_w[2] = 0;
    fmrx_write_reg(fmrx_reg_w);
#else
    u16 chan;
    if(fmrx_band){
        rf_freq -= 76;
    }else{
        rf_freq -= 87;
    }
    rf_freq *=10;
    rf_freq +=0.5;
    chan = (u16)rf_freq;
//	chan = 0x1C;
    // 浱ǰƵ
    fmrx_reg_w[3] = (chan >> 2) & 0xff;
    fmrx_reg_w[4] = (u8)(chan << 6) ;
    fmrx_reg_w[4] |= (1 << 4);      //г

    // ڲԶѰ̨ʹ
    fmrx_reg_w[1] &=  ~(1 << 0);      //SEEKλ1
    fmrx_write_reg(fmrx_reg_w);
    delay(10);
    fmrx_read_reg(fmrx_reg_r);
#endif
    return fmrx_get_freq();
}

void fmrx_reg_init(void)
{
    /** High byte first */

    /** 02H */
    fmrx_reg_w[1] = 0xd0;
    fmrx_reg_w[2] = 0x00;

    /** 03H */
    fmrx_reg_w[3] = 0x00;
    fmrx_reg_w[4] = 0x00;

    /** 04H */
    fmrx_reg_w[5] = 0x00;
    fmrx_reg_w[6] = 0x40;

    /** 05H */
    fmrx_reg_w[7] = 0x87;
    fmrx_reg_w[8] = 0xB8;

    fmrx_band = 0;
}

void fmrx_power(void)
{
    fmrx_reg_init();
    delay(500);

    // λָ
    fmrx_reg_w[1] = 0x00;
    fmrx_reg_w[2] = 0x02;
    fmrx_write_reg(fmrx_reg_w);

    delay(500);

    // ģĬϲ
    fmrx_reg_w[1] = 0xd0;
    fmrx_reg_w[2] = 0x01;
    fmrx_write_reg(fmrx_reg_w);
}

float fmrx_seek(u8 dir)
{
    u16 chan;

    if(dir)
    {
        fmrx_reg_w[1] |=  (1 << 1);
    }
    else
    {
        fmrx_reg_w[1] &=  ~(1 << 1);
    }

    fmrx_reg_w[4] &= ~(1 << 4);      //г

    // ڲԶѰ̨ʹ
    fmrx_reg_w[1] |=  (1 << 0);      //SEEKλ1

    chan = 0;
    chan = fmrx_reg_w[3];
    chan <<=8;
    chan |= fmrx_reg_w[4];
//    Serial.print("CUR CH:");
//    Serial.println(chan, HEX);

    fmrx_write_reg(fmrx_reg_w);
    delay(10);
    fmrx_read_reg(fmrx_reg_r);
    // ȴSTC ־λ
    while(0 == (fmrx_reg_r[1] & 0x40))
    {
        delay(50);
        // ȡڲ״̬
        fmrx_read_reg(fmrx_reg_r);
    }
    // ȡǰƵ
    chan = (fmrx_reg_r[1] & 0x03);
    chan <<= 8;
    chan |= fmrx_reg_r[2];
    chan = chan << 6;
//    Serial.print("CH:");
//    Serial.println(chan, HEX);
    // 浱ǰƵ
    fmrx_reg_w[3] = (chan >> 8) & 0xff;
//   fmrx_reg_w[3] = (chan >> 8) & 0xff;
    fmrx_reg_w[4] = ((u8)chan & 0xff);
//   fmrx_reg_w[4] = (chan & 0xff);

    return fmrx_get_freq();
}

float fmrx_get_freq()
{
    float ch;
    u16 chan;
    chan = (fmrx_reg_r[1] & 0x03);
    chan <<= 8;
    chan |= fmrx_reg_r[2];
    if(fmrx_band){
        ch = 76+chan*0.1;
    }else{
        ch = 87+chan*0.1;
    }
    return ch;
}
    void fmrx_set_volume(u8 vol)
{
    fmrx_reg_w[1] = 0xd0;
    fmrx_reg_w[2] = 0x01;

    fmrx_reg_w[4] &= ~(1 << 4);
    fmrx_reg_w[8] = (fmrx_reg_w[8]&0xF0) | (vol&0x0F);

    fmrx_write_reg(fmrx_reg_w);
}

void fmrx_set_rssi(u8 rssi)
{
    fmrx_reg_w[1] = 0xd0;
    fmrx_reg_w[2] = 0x01;

    fmrx_reg_w[4] &= ~(1 << 4);

    fmrx_reg_w[7] = (fmrx_reg_w[7]&0x80) | (rssi&0x7F);
    fmrx_write_reg(fmrx_reg_w);
}

void fmrx_select_band(u8 band)
{
    band &=0x0C;
    switch(band)
    {
    case 0x00:
        fmrx_band = 0;
        break;
    case 0x04:
    case 0x08:
        fmrx_band = 1;
        break;
    case 0x0C:
        return;
        break;
    }
    fmrx_reg_w[1] = 0xd0;
    fmrx_reg_w[2] = 0x01;

    fmrx_reg_w[4] &= ~(1 << 4);

    fmrx_reg_w[4] = (fmrx_reg_w[4]&0xF3)|band;
    fmrx_write_reg(fmrx_reg_w);
}
